package api;

import globalization.Translator;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.extensions.Extension;
import io.swagger.v3.oas.annotations.extensions.ExtensionProperty;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import javax.servlet.http.HttpServletRequest;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;

import repository.Repository;
import repository.RepositoryManager;

@Path("blocks")
@Produces({MediaType.APPLICATION_JSON, MediaType.TEXT_PLAIN})
@OpenAPIDefinition(
	extensions = @Extension(name = "translation", properties = {
		@ExtensionProperty(name="path", value="/Api/BlocksResource")
	})
)
public class BlocksResource {

	@Context
	HttpServletRequest request;

	private ApiErrorFactory apiErrorFactory;

	public BlocksResource() {
		this(new ApiErrorFactory(Translator.getInstance()));
	}

	public BlocksResource(ApiErrorFactory apiErrorFactory) {
		this.apiErrorFactory = apiErrorFactory;
	}

	@GET
	@Operation(
		description = "returns an array of the 50 last blocks generated by your accounts",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the blocks",
				//content = @Content(schema = @Schema(implementation = ???))
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "wallet does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="201")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/201")
					})
				}
			)
		}
	)
	public String getBlocks() {
		Security.checkApiCallAllowed("GET blocks", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/address/{address}")
	@Operation(
		description = "returns an array of the 50 last blocks generated by a specific address in your wallet",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET address:address"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the blocks",
				//content = @Content(schema = @Schema(implementation = ???)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "400",
				description = "invalid address",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="102")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/102")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "wallet does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="201")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/201")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "address does not exist in wallet",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="202")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/202")
					})
				}
			)
		}
	)
	public String getBlocks(@PathParam("address") String address) {
		Security.checkApiCallAllowed("GET blocks/address/" + address, request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/{signature}")
	@Operation(
		description = "returns the block that matches the given signature",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET signature"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the block",
				//content = @Content(schema = @Schema(implementation = ???)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "400",
				description = "invalid signature",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="101")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/101")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "block does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="301")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/301")
					})
				}
			)
		}
	)
	public String getBlock(@PathParam("signature") String signature) {
		Security.checkApiCallAllowed("GET blocks", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/first")
	@Operation(
		description = "returns the genesis block",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET first"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the block",
				//content = @Content(schema = @Schema(implementation = ???)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			)
		}
	)
	public String getFirstBlock() {
		Security.checkApiCallAllowed("GET blocks/first", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/last")
	@Operation(
		description = "returns the last valid block",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET last"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the block",
				//content = @Content(schema = @Schema(implementation = ???)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			)
		}
	)
	public String getLastBlock() {
		Security.checkApiCallAllowed("GET blocks/last", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/child/{signature}")
	@Operation(
		description = "returns the child block of the block that matches the given signature",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET child:signature"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the block",
				//content = @Content(schema = @Schema(implementation = ???)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "400",
				description = "invalid signature",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="101")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/101")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "block does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="301")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/301")
					})
				}
			)
		}
	)
	public String getChild(@PathParam("signature") String signature) {
		Security.checkApiCallAllowed("GET blocks/child", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/generatingbalance")
	@Operation(
		description = "calculates the generating balance of the block that will follow the last block",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET generatingbalance"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the generating balance",
				content = @Content(schema = @Schema(implementation = long.class)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			)
		}
	)
	public long getGeneratingBalance() {
		Security.checkApiCallAllowed("GET blocks/generatingbalance", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/generatingbalance/{signature}")
	@Operation(
		description = "calculates the generating balance of the block that will follow the block that matches the signature",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET generatingbalance:signature"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the block",
				content = @Content(schema = @Schema(implementation = long.class)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "400",
				description = "invalid signature",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="101")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/101")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "block does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="301")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/301")
					})
				}
			)
		}
	)
	public long getGeneratingBalance(@PathParam("signature") String signature) {
		Security.checkApiCallAllowed("GET blocks/generatingbalance", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/time")
	@Operation(
		description = "calculates the time it should take for the network to generate the next block",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET time"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the time", // in seconds?
				content = @Content(schema = @Schema(implementation = long.class)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			)
		}
	)
	public long getTimePerBlock() {
		Security.checkApiCallAllowed("GET blocks/time", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/time/{generatingbalance}")
	@Operation(
		description = "calculates the time it should take for the network to generate blocks when the current generating balance in the network is the specified generating balance",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET time:generatingbalance"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the time", // in seconds?
				content = @Content(schema = @Schema(implementation = long.class)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			)
		}
	)
	public String getTimePerBlock(@PathParam("generating") long generatingbalance) {
		Security.checkApiCallAllowed("GET blocks/time", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/height")
	@Operation(
		description = "returns the block height of the last block.",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET height"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the height",
				content = @Content(schema = @Schema(implementation = int.class)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			)
		}
	)
	public int getHeight() {
		Security.checkApiCallAllowed("GET blocks/height", request);

        try (final Repository repository = RepositoryManager.getRepository()) {
            return repository.getBlockRepository().getBlockchainHeight();
        } catch (Exception e) {
            throw this.apiErrorFactory.createError(ApiError.UNKNOWN, e);
        }
	}

	@GET
	@Path("/height/{signature}")
	@Operation(
		description = "returns the block height of the block that matches the given signature",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET height:signature"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the height",
				content = @Content(schema = @Schema(implementation = int.class)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "400",
				description = "invalid signature",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="101")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/101")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "block does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="301")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/301")
					})
				}
			)
		}
	)
	public int getHeight(@PathParam("signature") String signature) {
		Security.checkApiCallAllowed("GET blocks/height", request);

		throw new UnsupportedOperationException();
	}

	@GET
	@Path("/byheight/{height}")
	@Operation(
		description = "returns the block whith given height",
		extensions = @Extension(name = "translation", properties = {
			@ExtensionProperty(name="path", value="GET byheight:height"),
			@ExtensionProperty(name="description.key", value="operation:description")
		}),
		responses = {
			@ApiResponse(
				description = "the block",
				//content = @Content(schema = @Schema(implementation = ???)),
				extensions = {
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="success_response:description")
					})
				}
			),
			@ApiResponse(
				responseCode = "422",
				description = "block does not exist",
				content = @Content(schema = @Schema(implementation = ApiErrorMessage.class)),
				extensions = {
					@Extension(properties = {
						@ExtensionProperty(name="apiErrorCode", value="301")
					}),
					@Extension(name = "translation", properties = {
						@ExtensionProperty(name="description.key", value="ApiError/301")
					})
				}
			)
		}
	)
	public String getbyHeight(@PathParam("height") int height) {
		Security.checkApiCallAllowed("GET blocks/byheight", request);

		throw new UnsupportedOperationException();
	}
}
